import sys
import logging
import json
import requests
import pprint
from scripts import cerebro_spring_lib
from scripts import common_lib
import jwt
import datetime
import os.path

CSP_URI = sys.argv[1]
REQUEST_JSON_FILE = sys.argv[2]
NAMESPACE = sys.argv[3]
SERVICE_NAME = sys.argv[4]
SERVICE_PORT = sys.argv[5]
CSP_CLIENT_ID = sys.argv[6]
CSP_CLIENT_SECRET = sys.argv[7]
HEADER_TYPE = sys.argv[8]
SM_CONTEXT = sys.argv[9] or "cloud"
print("SM_CONTEXT is :", SM_CONTEXT)
if SM_CONTEXT == "sm-hub":
    # port needs to be rechecked to point to 8000
    SERVICE_PORT = "8080"
    SM_EBALED = True
    orgID = ""
    file_exist = os.path.isfile("/opt/le-mans/sm/smConfigFile")
    if file_exist:
        with open("/opt/le-mans/sm/smConfigFile", "r") as f:
            for line in f:
                non_empty = line.strip()
                if non_empty:
                    key, value = line.strip().split("=")
                    if key == "defaultOrgId":
                        orgID = value.strip().strip('"')
                        print("defaultOrgId is : ", orgID)

        if not orgID:
            print("defaultOrgId not found in the file")
    else:
        print("/opt/le-mans/sm/smConfigFile file not found")

    now = datetime.datetime.utcnow()

    private_key_pem = ""
    file_exist = os.path.isfile("/run/auth/ssl/private.pem")
    if file_exist:
        with open("/run/auth/ssl/private.pem", "r") as f:
            private_key_pem = f.read()
    else:
        print("private_key.pem file not found in the pod")

    print("reading the file from /run/auth/ssl/private.pem :", private_key_pem)

    payload = {
        "iat": now,
        "exp": now + datetime.timedelta(hours=1),
        "org_id": orgID,
        "context_name": orgID,
        "sub": "1234567890",
        "issuer": "le-mans-gateway",
    }
    # Algorithm to use for signing
    algorithm = "RS256"

    # Create the token
    jwttoken = jwt.encode(payload, private_key_pem, algorithm=algorithm)

    print("JWT Token:", jwttoken)
    logging.info("%s JWT Token:", jwttoken)
    jwttoken = "Bearer " + jwttoken
    print("Bearer JWT Token:", jwttoken)
else:
    SM_EBALED = False
logging.getLogger().setLevel(logging.DEBUG)


class AuthenticatingAdapter(requests.adapters.HTTPAdapter):
    """Injects a csp token and proxies the request"""

    def __init__(self, token_cb):
        self.token = None
        self.token_cb = token_cb
        self.session = requests.Session()
        super().__init__()

    def send(self, request, **kwargs):
        if self.token is None and self.token_cb:
            self.token = self.token_cb()
            logging.info("Injected token=%s", self.token)

        if HEADER_TYPE == "x-xenon-auth-token":
            request.headers["x-xenon-auth-token"] = self.token
        else:
            request.headers["Authorization"] = "Bearer " + self.token

        if request.method in ("POST", "PUT", "PATCH"):
            kwargs["data"] = request.body

        debug_info = {
            "url": request.url,
            "headers": request.headers,
            "method": request.method,
            "data": request.body,
        }
        logging.info(pprint.pformat(debug_info))

        req_executor = getattr(self.session, request.method.lower())
        return req_executor(request.url, headers=request.headers, **kwargs)


def _get_session_with_cerebro_proxy_adapter():
    with requests.Session() as requests_session:

        def get_token():
            return cerebro_spring_lib.get_csp_auth_token(
                requests_session,
                CSP_URI,
                CSP_CLIENT_ID,
                CSP_CLIENT_SECRET,
            )

    adapter = AuthenticatingAdapter(get_token)
    return cerebro_spring_lib.cerebro_session(adapter)


def create(namespace, csp_uri, service_name, service_port):
    service_exists = common_lib.check_if_service_exists(
        namespace, service_name, service_port
    )
    if not service_exists:
        logging.info("Could not find the service. Skipping .")
        return None
    service_hostname = common_lib.get_service_hostname(namespace, service_name)
    service_adapter = _get_session_with_cerebro_proxy_adapter()
    return requestHelper(service_adapter, service_hostname, service_name, service_port)


class requestHelper:
    def __init__(self, request_adapter, service_hostname, service_name, service_port):
        self.request_adapter = request_adapter
        self.service_hostname = service_hostname
        self.service_name = service_name
        self.service_port = service_port

    def call_api(self, REQUEST_TYPE, REQUEST_API, REQUEST_DATA=None):
        url = "http://" + self.service_name + ":" + self.service_port + REQUEST_API
        result = requests.Session()
        result.mount("http://", self.request_adapter)
        if REQUEST_TYPE == "POST":
            response = result.post(url, data=json.dumps(REQUEST_DATA))
        elif REQUEST_TYPE == "GET":
            response = result.get(url)
        elif REQUEST_TYPE == "PUT":
            response = result.put(url, data=json.dumps(REQUEST_DATA))
        else:
            response = result.delete(url)
        response.raise_for_status()
        logging.info("called api  %s", url)
        logging.info(response)


if __name__ == "__main__":
    with open(REQUEST_JSON_FILE) as request_json_file:
        request_json = json.load(request_json_file)
        if SM_EBALED:
            logging.info("inside SM world")
            try:
                logging.info(
                    "http://"
                    + SERVICE_NAME
                    + "."
                    + NAMESPACE
                    + ".svc.cluster.local:"
                    + SERVICE_PORT
                )
                requests.get(
                    "http://"
                    + SERVICE_NAME
                    + "."
                    + NAMESPACE
                    + ".svc.cluster.local:"
                    + SERVICE_PORT
                )
                sm_result = requests.Session()
                sm_url = (
                    "http://"
                    + SERVICE_NAME
                    + ":"
                    + SERVICE_PORT
                    + "/management/bootstrap"
                )
                sm_response = sm_result.post(
                    sm_url, data={}, headers={"Authorization": jwttoken}
                )
                logging.debug("%s is the sm_url ", sm_url)
                logging.debug("%s is the sm_response ", sm_response)
            except Exception:
                logging.info(
                    "Could not initiate the SM service. Please check the resource service"
                )
        else:
            requestobject = create(NAMESPACE, CSP_URI, SERVICE_NAME, SERVICE_PORT)
            if requestobject is None:
                logging.warning("Skipping requests as none found")
                exit(0)
            for service, request_content in request_json.items():
                for i in range(0, len(request_content)):
                    logging.info(
                        "%s call on %s",
                        request_content[i]["REQUEST_TYPE"],
                        request_content[i]["REQUEST_API"],
                    )
                    if "REQUEST_DATA" in request_content[i]:
                        requestobject.call_api(
                            request_content[i]["REQUEST_TYPE"],
                            request_content[i]["REQUEST_API"],
                            request_content[i]["REQUEST_DATA"],
                        )
                    else:
                        requestobject.call_api(
                            request_content[i]["REQUEST_TYPE"],
                            request_content[i]["REQUEST_API"],
                        )
